from google.cloud import tasks_v2
from google.protobuf import timestamp_pb2
import json
import datetime


class Task:
    def __init__(self):
        self.status = None
        self.fn_url = None
        self.task_name = None
        self.payload = None
        self.delay = None
        self.queue = None


    def post(fn_url, task_name, payload=None, delay = None, queue = 'default'):
        """ 
        Create a new google cloud task

        Provided a url and payload, schedule a task to be executed now or at a later date by puttin it in a google task queue. 

        Parameters
        ----------
        fn_url (str): The url of the function to execute
        task_name (str): The name of the task, used for deduplication
        payload (dict): The payload of the post request
        delay (int): The delay to start the task in seconds
        queue (str): The queue to use for the task (defaults to "default")

        Returns
        -------
        bool: Task success
        """

        
        # Create a client.
        client = tasks_v2.CloudTasksClient.from_service_account_json('key.json')

        # Assign config values
        project = 'kraken-v1'
        location = 'us-central1'
        in_seconds = 1


        # Construct the fully qualified queue name.
        parent = client.queue_path(project, location, queue)
        task_name = 'projects/' + project + '/locations/' + location + '/queues/' + queue + '/tasks/' + task_name


        # Build task
        if payload is not None:
            # Construct post request

            task = {
                'http_request': {  # Specify the type of request.
                    'http_method': 'POST',
                    'headers': {'Content-type': 'application/json'},
                    'url': fn_url,
                    },
                'name': task_name

                }

            # The API expects a payload of type bytes.
            #converted_payload = payload.encode()
            converted_payload = json.dumps(payload, default=str)
            converted_payload = converted_payload.encode()

            # Add the payload to the request.
            task['http_request']['body'] = converted_payload

        else:
            # Construct get request
            task = {
                'http_request': {  # Specify the type of request.
                    'http_method': 'GET',
                    'url': fn_url,
                    },
                'name': task_name

                }

        # Establish time delay
        if in_seconds is not None:
            # Convert "seconds from now" into an rfc3339 datetime string.
            d = datetime.datetime.utcnow() + datetime.timedelta(seconds=in_seconds)

            # Create Timestamp protobuf.
            timestamp = timestamp_pb2.Timestamp()
            timestamp.FromDatetime(d)

            # Add the timestamp to the tasks.
            task['schedule_time'] = timestamp


        # Send the task
        response = client.create_task(request = {'parent': parent, 'task': task})

        return response



    def get(self, task_name): 
        """ 
        Get the status o f a task

        Parameters
        ----------
        task_name (str): The name of the task

        Returns
        -------
        str: Status
        """

        a=1



    def get_all(self, queue): 
        """ 
        Get the list of active tasks in a queue
        Parameters
        ----------
        queue (str): The name of the queue

        Returns
        -------
        str: Status
        """

        a=1


"""

Reference from : https://cloud.google.com/storage/docs/listing-objects#storage-list-objects-python


"""





from google.cloud import storage
import uuid
import json

class Storage:

    def __init__(self):
        a=1


    def get(self, bucket_name, source_blob_name, destination_file_name):
        """Downloads a blob from the bucket."""
        # bucket_name = "your-bucket-name"
        # source_blob_name = "storage-object-name"
        # destination_file_name = "local/path/to/file"

        storage_client = storage.Client()

        bucket = storage_client.bucket(bucket_name)
        blob = bucket.blob(source_blob_name)
        blob.download_to_filename(destination_file_name)

        print(
            "Blob {} downloaded to {}.".format(
                source_blob_name, destination_file_name
            )
        )




    def post(self, bucket, record):
        """Uploads a file to the bucket."""
        # bucket_name = "your-bucket-name"
        # source_file_name = "local/path/to/file"
        
        blob_name = uuid.uuid4().hex

        storage_client = storage.Client()
        bucket = storage_client.bucket(bucket)
        blob = bucket.blob(blob_name)

        blob.upload_from_string(record)

        new_record = {}
        new_record['name'] = blob.name
        new_record['bucket_name']= blob.bucket.name
        new_record['storage_class'] = blob.storage_class
        new_record['id'] = blob.id
        new_record['size'] = blob.size
        new_record['updated'] = blob.updated
        new_record['generation'] = blob.generation
        new_record['metadata'] = blob.metadata

        return new_record
        

    def delete(self, bucket_name, blob_name):
        """Deletes a blob from the bucket."""
        # bucket_name = "your-bucket-name"
        # blob_name = "your-object-name"

        storage_client = storage.Client()

        bucket = storage_client.bucket(bucket_name)
        blob = bucket.blob(blob_name)
        blob.delete()

        print("Blob {} deleted.".format(blob_name))



    def create_bucket_class_location(self, bucket_name):
        """Create a new bucket in specific location with storage class"""
        # bucket_name = "your-new-bucket-name"

        storage_client = storage.Client()

        bucket = storage_client.bucket(bucket_name)
        bucket.storage_class = "COLDLINE"
        new_bucket = storage_client.create_bucket(bucket, location="us")

        print(
            "Created bucket {} in {} with storage class {}".format(
                new_bucket.name, new_bucket.location, new_bucket.storage_class
            )
        )
        return new_bucket

        
    def list_blobs(bucket_name):
        """Lists all the blobs in the bucket."""
        # bucket_name = "your-bucket-name"

        storage_client = storage.Client()

        # Note: Client.list_blobs requires at least package version 1.17.0.
        blobs = storage_client.list_blobs(bucket_name)

        for blob in blobs:
            print(blob.name)



from google.cloud import bigquery
import datetime
import json


class Bigquery:

    def get(self):
    
        # Construct a BigQuery client object.
        client = bigquery.Client()

        query = """
            SELECT record_type, COUNT(record_type) as total_record, MAX(createdDate) as last_updated 
            FROM `kraken-v1.kraken.schema_records`
            GROUP BY record_type
            ORDER BY total_record DESC
            LIMIT 20
        """
        query_job = client.query(query)  # Make an API request.

        records = []

        for row in query_job:
            record = {}
            record['record_type'] = row['record_type']
            record['count'] = row['total_record']
            record['last_updated'] = row['last_updated']
            records.append(record)

        return json.dumps(records, default=str)




    def post(self, record_type, record_id, schema_name, schema_email, schema_url, created_date):

        # Construct a BigQuery client object.
        client = bigquery.Client()

        # TODO(developer): Set table_id to the ID of the model to fetch.
        table_id = "kraken-v1.kraken.kraken_record"

        table = client.get_table(table_id)  # Make an API request.

        rows_to_insert = [(record_type, record_id, schema_name, schema_email, schema_url, created_date)]


        errors = client.insert_rows(table, rows_to_insert)  # Make an API request.
        if errors == []:
            print("New rows have been added.")
        else:
            print("Error")
